// Copyright 2016 the Go-FUSE Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package splice

import (
	"fmt"
	"log"
	"os"
	"syscall"

	"golang.org/x/sys/unix"
)

func (p *Pair) LoadFromAt(fd uintptr, sz int, off int64) (int, error) {
	n, err := syscall.Splice(int(fd), &off, p.w, nil, sz, 0)
	return int(n), err
}

func (p *Pair) LoadFrom(fd uintptr, sz int) (int, error) {
	if sz > p.size {
		return 0, fmt.Errorf("LoadFrom: not enough space %d, %d",
			sz, p.size)
	}

	n, err := syscall.Splice(int(fd), nil, p.w, nil, sz, 0)
	if err != nil {
		err = os.NewSyscallError("Splice load from", err)
	}
	return int(n), err
}

func (p *Pair) WriteTo(fd uintptr, n int) (int, error) {
	m, err := syscall.Splice(p.r, nil, int(fd), nil, int(n), 0)
	if err != nil {
		err = os.NewSyscallError("Splice write", err)
	}
	return int(m), err
}

const (
	_SPLICE_F_NONBLOCK = 0x2
	_FIONREAD          = 0x541b
)

func (p *Pair) discard() {
	for {
		_, err := syscall.Splice(p.r, nil, devNullFD(), nil, int(p.size), _SPLICE_F_NONBLOCK)
		if err != nil && err != syscall.EAGAIN {
			errR := syscall.Close(p.r)
			errW := syscall.Close(p.w)

			// This can happen if something closed our fd
			// inadvertently (eg. double close)
			log.Panicf("splicing into /dev/null: %v (close R %d '%v', close W %d '%v')", err, p.r, errR, p.w, errW)
		}

		// Verify the pipe is empty before returning it to the pool.
		n, err := unix.IoctlGetInt(p.r, _FIONREAD)
		if err != nil {
			log.Panicf("FIONREAD on pipe: %v", err)
		}
		if n == 0 {
			return
		}
	}
}
